define([
	"../scripts/services/running-game-service.js",
	"../scripts/services/extensions-service.js",
	"../scripts/services/windows-service.js",
	"../scripts/services/hotkeys-service.js",
	"../scripts/services/gep-service.js",
	"../scripts/services/event-bus.js"
], function (
	runningGameService,
	extensionsService,
	WindowsService,
	hotkeysService,
	gepService,
	eventBus
) {
	
	class BackgroundController {
				
		static async run() {
			// Used to transfer game events and info updates
			window.ow_eventBus = eventBus;
			
			window.find_game_resolution = BackgroundController.find_game_resolution;
			window.move_window_to = BackgroundController.move_window_to;
			window.change_window_size = BackgroundController.change_window_size;
			
			await BackgroundController._restoreLaunchWindow();
						
			// Switch between desktop/in-game windows when launching/closing game
			runningGameService.addGameRunningChangedListener(
				BackgroundController._onRunningGameChanged
			);
			
			// Restore desktop/in-game window when clicking the dock icon (launching app that is already launched)
			extensionsService.addAppLaunchTriggeredListener(
				BackgroundController._onAppLaunchTriggered
			);
			
			// Listen to Tab hold for ESE
			hotkeysService.setESEHotkey(async (state) => {
				const ese_state = await WindowsService.getWindowState('blue');
				if ( state === "down" && (ese_state === "minimized" || ese_state == 'hidden') ) {
					WindowsService.restore('blue');
				} else if ( state === "up" && (ese_state === "normal" || ese_state === "maximized") ) {
					WindowsService.hide('blue');
				}
			});
		}
		
		/**
		 * Returns the current screen resolution in-game
		 * @public
		 */
		static find_game_resolution = function(callback) {
			overwolf.games.getRunningGameInfo(function(gameinfo){
				
				window.gameinfo = false;
				
				if(arguments.length == 1) {
					window.gameinfo = gameinfo;
				} else {
					$.each(gameinfo, function(k,v){
						if(typeof v.id === 'undefined') return true;
						if(v.classId == 10826) {
							window.gameinfo = v;
							return false;
						}
					});
				}
				
				if(typeof window.gameinfo === 'undefined' || window.gameinfo === false || window.gameinfo === null) {
					var sw = 1920; // Screen Width
					var sh = 1080; // Screen Height
				} else {
					var sw = window.gameinfo.logicalWidth; // Screen Width
					var sh = window.gameinfo.logicalHeight; // Screen Height
				}
				
				var result = {
					"width": sw,
					"height": sh
				};
				
				callback(result);
			});
		}
		
		/**
		 * Move a window
		 * @public
		 */
		static move_window_to(window, left, top) {
			console.log('Moving window '+window+' to left:'+left+' top:'+top);
			var result = overwolf.windows.changePosition(window, Math.round(left), Math.round(top));
			return result;
		}
		
		/**
		 * Change a window's size
		 * @public
		 */
		static change_window_size(window, width, height) {
			console.log('Changing window size for '+window+' to width:'+width+' height:'+height);
			var result = overwolf.windows.changeSize(window, Math.round(width), Math.round(height));
			return result;
		}
		
		/**
		 * Handle game opening/closing
		 * @private
		 */
		static async _onRunningGameChanged(isGameRunning) {
			if (isGameRunning) {
				// Register to game events
				gepService.registerToGEP(
					BackgroundController.onGameEvents,
					BackgroundController.onInfoUpdate
				);
				
				console.log('game launch detected');
								
				// Starting BLUE window, but hidden
				WindowsService.hide('blue');
				
			} else {
				// Close windows
				WindowsService.close('blue');
			}
		}

		/**
		 * Open the relevant window on app/game launch
		 * @private
		 */
		static async _restoreLaunchWindow() {
			
			const isGameRunning = await runningGameService.isGameRunning();

			if (!isGameRunning) {
				
			} else {
				console.log('game launch detected');
								
				// Starting BLUE window, but hidden
				WindowsService.hide('blue');
				
				gepService.registerToGEP(
					BackgroundController.onGameEvents,
					BackgroundController.onInfoUpdate
				);
			}
		}

		/**
		 * Open the relevant window on dock icon press when app already launched
		 * @private
		 */
		static async _onAppLaunchTriggered() {
			const isGameRunning = await runningGameService.isGameRunning();
			if (!isGameRunning) {
				
			} else {
				console.log('manual launch detected');
								
				// Starting BLUE window, but hidden
				WindowsService.hide('blue');
				
				gepService.registerToGEP(
					BackgroundController.onGameEvents,
					BackgroundController.onInfoUpdate
				);
			}
		}
		
		/**
		 * Pass events to windows that are listening to them
		 * @private
		 */
		static onGameEvents(data) {
			data.events.forEach(event => {
				window.ow_eventBus.trigger("event", event);
			});
		}

		/**
		 * Pass info updates to windows that are listening to them
		 * @private
		 */
		static onInfoUpdate(data) {
			window.ow_eventBus.trigger("info", data);
		}
		
	}

	return BackgroundController;
});
